package xyz.zhouxy.plusone.system.application.service;

import xyz.zhouxy.plusone.system.constant.AuthLogic;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.dev33.satoken.stp.StpLogic;
import cn.hutool.core.lang.Assert;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalUtil;
import xyz.zhouxy.plusone.system.application.exception.AccountLoginException;
import xyz.zhouxy.plusone.system.application.query.AccountQueries;
import xyz.zhouxy.plusone.system.application.query.result.AccountDetails;
import xyz.zhouxy.plusone.system.application.query.result.MenuViewObject;
import xyz.zhouxy.plusone.system.application.service.command.ChangePasswordByOtpCommand;
import xyz.zhouxy.plusone.system.application.service.command.ChangePasswordCommand;
import xyz.zhouxy.plusone.system.application.service.command.ChangePasswordWithoutLoginCommand;
import xyz.zhouxy.plusone.system.domain.model.account.Account;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Principal;
import xyz.zhouxy.plusone.validator.InvalidInputException;

/**
 * 账号对当前帐号进行操作
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
public class AccountContextService {

    private final static StpLogic adminAuthLogic = AuthLogic.adminAuthLogic;

    @Resource
    private AccountQueries accountQueries;
    @Resource
    private MenuManagementService menuManagementService;
    @Resource
    private AccountRepository accountRepository;
    @Resource
    private MailAndSmsVerifyService mailAndSmsVerifyService;

    public AccountDetails getAccountInfo() {
        adminAuthLogic.checkLogin();
        long accountId = adminAuthLogic.getLoginIdAsLong();
        return accountQueries.queryAccountDetails(accountId);
    }

    public void logout() {
        adminAuthLogic.checkLogin();
        adminAuthLogic.logout();
    }

    public List<MenuViewObject> getMenuTree() {
        adminAuthLogic.checkLogin();
        long accountId = adminAuthLogic.getLoginIdAsLong();
        return menuManagementService.queryByAccountId(accountId);
    }

    @Transactional
    public void changePassword(ChangePasswordCommand command) {
        adminAuthLogic.checkLogin();
        Account account = accountRepository.find(adminAuthLogic.getLoginIdAsLong());
        account.checkPassword(command.getPassword());
        account.changePassword(command.getNewPassword(), command.getPasswordConfirmation());
        accountRepository.save(account);
        adminAuthLogic.logout();
    }

    @Transactional
    public void changePasswordWithoutLogin(ChangePasswordWithoutLoginCommand command) {
        String principal = command.getAccount();
        Principal emailOrMobilePhone = PrincipalUtil.getEmailOrMobilePhone(principal);

        Account account = emailOrMobilePhone instanceof Email
                ? accountRepository.findByEmail((Email) emailOrMobilePhone)
                : accountRepository.findByMobilePhone((MobilePhone) emailOrMobilePhone);
        account.checkPassword(command.getOldPassword());
        account.changePassword(command.getNewPassword(), command.getPasswordConfirmation());
        accountRepository.save(account);
        adminAuthLogic.logout();
    }

    @Transactional
    public void changePasswordByOtp(ChangePasswordByOtpCommand command) {

        var principal = command.getAccount();
        Account account = switch (command.getPrincipalType()) {
            case EMAIL -> accountRepository.findByEmail(Email.of(principal));
            case MOBILE_PHONE -> accountRepository.findByMobilePhone(MobilePhone.of(principal));
            default -> throw InvalidInputException.unsupportedPrincipalTypeException("输入邮箱地址或手机号");
        };
        Assert.notNull(account, () -> AccountLoginException.accountNotExistException());

        mailAndSmsVerifyService.checkOtp(principal, command.getOtp());
    }
}
