package xyz.zhouxy.plusone.system.application.service;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.hutool.core.lang.Assert;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalType;
import xyz.zhouxy.plusone.system.application.common.util.PrincipalUtil;
import xyz.zhouxy.plusone.system.application.exception.AccountLoginException;
import xyz.zhouxy.plusone.system.application.query.AccountQueries;
import xyz.zhouxy.plusone.system.application.query.result.LoginInfoViewObject;
import xyz.zhouxy.plusone.system.application.service.command.LoginByOtpCommand;
import xyz.zhouxy.plusone.system.application.service.command.LoginByPasswordCommand;
import xyz.zhouxy.plusone.system.domain.model.account.Account;
import xyz.zhouxy.plusone.system.domain.model.account.AccountRepository;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Username;
import xyz.zhouxy.plusone.validator.InvalidInputException;
import xyz.zhouxy.plusone.validator.ValidateDto;

/**
 * Admin 账号登录
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class AdminLoginService {

    private final AccountRepository accountRepository;
    private final AccountQueries accountQueries;
    private final MailAndSmsVerifyService mailAndSmsVerifyService;

    AdminLoginService(AccountRepository accountRepository, AccountQueries accountQueries,
            MailAndSmsVerifyService mailAndSmsVerifyService) {
        this.accountRepository = accountRepository;
        this.accountQueries = accountQueries;
        this.mailAndSmsVerifyService = mailAndSmsVerifyService;
    }

    @ValidateDto
    public LoginInfoViewObject loginByPassword(LoginByPasswordCommand command) {
        var principal = command.getPrincipal();
        Account account = switch (command.getPrincipalType()) {
            case USERNAME -> accountRepository.findByUsername(Username.of(principal));
            case EMAIL -> accountRepository.findByEmail(Email.of(principal));
            case MOBILE_PHONE -> accountRepository.findByMobilePhone(MobilePhone.of(principal));
        };
        Assert.notNull(account, () -> AccountLoginException.accountNotExistException());
        var isPasswordCorrect = account.checkPassword(command.getPassword());
        Assert.isTrue(isPasswordCorrect, () -> AccountLoginException.passwordErrorException());

        adminAuthLogic.login(account.getId().orElseThrow(), command.isRememberMe());
        var accountDetails = accountQueries.queryAccountDetails(account.getId().orElseThrow());
        return LoginInfoViewObject.of(adminAuthLogic.getTokenValue(), accountDetails);
    }

    @ValidateDto
    public LoginInfoViewObject loginByOtp(LoginByOtpCommand command) {
        var principal = command.getPrincipal();
        Account account = switch (command.getPrincipalType()) {
            case EMAIL -> accountRepository.findByEmail(Email.of(principal));
            case MOBILE_PHONE -> accountRepository.findByMobilePhone(MobilePhone.of(principal));
            default -> throw InvalidInputException.unsupportedPrincipalTypeException("输入邮箱地址或手机号");
        };
        Assert.notNull(account, () -> AccountLoginException.accountNotExistException());

        mailAndSmsVerifyService.checkOtp(principal, command.getOtp());

        adminAuthLogic.login(account.getId().orElseThrow(), command.isRememberMe());

        var accountDetails = accountQueries.queryAccountDetails(account.getId().orElseThrow());
        return LoginInfoViewObject.of(adminAuthLogic.getTokenValue(), accountDetails);
    }

    public void sendOtp(String principal) {
        PrincipalType principalType = PrincipalUtil.getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            mailAndSmsVerifyService.sendOtpToEmail(Email.of(principal));
        } else {
            mailAndSmsVerifyService.sendOtpToMobilePhone(MobilePhone.of(principal));
        }
    }
}
